@@ -22,6 +22,7 @@
 #include <ggv-control.h>
 #include <gsdefaults.h>
 #include <ggvutils.h>
+#include <ggv-document.h>
 
 struct _GgvControlPrivate {
 	GgvPostScriptView *ps_view;
@@ -100,27 +101,6 @@ ggv_control_finalize (GObject *object)
 	G_OBJECT_CLASS (ggv_control_parent_class)->finalize (object);
 }
 
-static gboolean
-is_auto_fit_set(GgvControl *control)
-{
-	GgvPostScriptView *ps_view;
-	GtkWidget *gs;
-
-	ps_view = control->priv->ps_view;
-	gs = ggv_postscript_view_get_widget(ps_view);
-
-	switch(gtk_gs_get_zoom_mode(GTK_GS(gs))) {
-	case GTK_GS_ZOOM_ABSOLUTE:
-		return FALSE;
-	case GTK_GS_ZOOM_FIT_WIDTH:
-	case GTK_GS_ZOOM_FIT_PAGE:
-		return TRUE;
-	default:
-		return FALSE;
-	}
-
-}
-
 static void
 sync_zoom_level_items(GgvControl *control)
 {
@@ -186,6 +166,95 @@ sync_command_state(GgvControl *control)
 	sync_zoom_level_items(control);
 }
 
+static void
+_set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolean sens)
+{
+	gchar *prop_val = sens?"1":"0";
+	Bonobo_UIContainer container;
+	gint i;
+
+	container = bonobo_ui_component_get_container (uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
+	bonobo_ui_component_freeze(uic, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomIn",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomOut",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToDefault",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFit",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFitWidth",
+								 "sensitive", prop_val, NULL);
+	for(i = 0; zoom_level_items[i].path != NULL; i++) {
+		bonobo_ui_component_set_prop(uic, zoom_level_items[i].path,
+									 "sensitive", prop_val, NULL);
+	}
+	bonobo_ui_component_thaw(uic, NULL);
+}
+
+static void
+set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	bonobo_ui_component_set_prop(control->priv->uic,
+								 "/Sidebar/Zoom Control Placeholder/ZoomControl",
+								 "sensitive", sens?"1":"0", NULL);
+	_set_zoom_items_sensitivity(control, control->priv->uic, sens);
+	if(control->priv->popup_uic != NULL)
+		_set_zoom_items_sensitivity(control, control->priv->popup_uic, sens);
+}
+
+static void
+set_auto_fit_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	gchar *prop_val = sens?"1":"0";
+	Bonobo_UIContainer container;
+
+	container = bonobo_ui_component_get_container (control->priv->uic);
+	if (container == CORBA_OBJECT_NIL)
+		return;
+
+	bonobo_ui_component_freeze(control->priv->uic, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitNone",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitWidth",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitPage",
+								 "sensitive", prop_val, NULL);
+	bonobo_ui_component_thaw(control->priv->uic, NULL);	
+}
+
+static gboolean
+is_auto_fit_set(GgvControl *control)
+{
+	GgvPostScriptView *ps_view;
+	GtkWidget *gs;
+
+	ps_view = control->priv->ps_view;
+	gs = ggv_postscript_view_get_widget(ps_view);
+
+	switch(gtk_gs_get_zoom_mode(GTK_GS(gs))) {
+	case GTK_GS_ZOOM_ABSOLUTE:
+		return FALSE;
+	case GTK_GS_ZOOM_FIT_WIDTH:
+	case GTK_GS_ZOOM_FIT_PAGE:
+		return TRUE;
+	default:
+		return FALSE;
+	}
+
+}
+
+static void
+set_command_items_sensitivity(GgvControl *control, gboolean sens)
+{
+	set_auto_fit_items_sensitivity(control, sens);
+	sens = sens & !is_auto_fit_set(control);
+	set_zoom_items_sensitivity(control, sens);
+}
+
 static void
 zoomable_set_frame_cb (BonoboZoomable *zoomable, GgvControl *control)
 {
@@ -324,46 +393,6 @@ sw_size_allocate(GtkWidget *widget, GtkAllocation *alloc, gpointer data)
 	gtk_gs_set_available_size(view, w, h);
 }
 
-static void
-_set_zoom_items_sensitivity(GgvControl *control, BonoboUIComponent *uic, gboolean sens)
-{
-	gchar *prop_val = sens?"1":"0";
-	Bonobo_UIContainer container;
-	gint i;
-
-	container = bonobo_ui_component_get_container (uic);
-	if (container == CORBA_OBJECT_NIL)
-		return;
-
-	bonobo_ui_component_freeze(uic, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomIn",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomOut",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomToDefault",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFit",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(uic, "/commands/ZoomToFitWidth",
-								 "sensitive", prop_val, NULL);
-	for(i = 0; zoom_level_items[i].path != NULL; i++) {
-		bonobo_ui_component_set_prop(uic, zoom_level_items[i].path,
-									 "sensitive", prop_val, NULL);
-	}
-	bonobo_ui_component_thaw(uic, NULL);
-}
-
-static void
-set_zoom_items_sensitivity(GgvControl *control, gboolean sens)
-{
-	bonobo_ui_component_set_prop(control->priv->uic,
-								 "/Sidebar/Zoom Control Placeholder/ZoomControl",
-								 "sensitive", sens?"1":"0", NULL);
-	_set_zoom_items_sensitivity(control, control->priv->uic, sens);
-	if(control->priv->popup_uic != NULL)
-		_set_zoom_items_sensitivity(control, control->priv->popup_uic, sens);
-}
-
 static void
 listener_ZoomLevel_cb(BonoboUIComponent *uic, const char *path,
 					  Bonobo_UIComponent_EventType type, const char *state,
@@ -430,6 +459,7 @@ listener_AutoFitMode_cb(BonoboUIComponent *uic, const char *path,
 		gtk_gs_set_zoom_mode(gs, GTK_GS_ZOOM_FIT_PAGE);
 		set_zoom_items_sensitivity(control, FALSE);
 	}
+
 	gtk_widget_unref(GTK_WIDGET(gs));
 }
 
@@ -462,6 +492,7 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 	GgvControl *control = GGV_CONTROL(data);
 	GtkGSOrientation orientation = gtk_gs_get_orientation(gs);
 	GgvPostScriptView *ps_view = control->priv->ps_view;
+	gfloat val;
 
 	/* ugh. the possibilities! */
 	switch (key) {
@@ -563,11 +594,16 @@ ggv_control_key_press_event(GtkWidget *widget, GdkEventKey *event,
 			ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
 		break;
 	case GDK_Page_Up:
-		ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) - 1);
+	case GDK_KP_Page_Up:
+		gtk_adjustment_set_value(gs->vadj, gs->vadj->value - gs->vadj->page_increment);
 		break;
 	case GDK_Page_Down:
-		ggv_postscript_view_goto_page(ps_view, gtk_gs_get_current_page(gs) + 1);
-		break;
+	case GDK_KP_Page_Down:
+		val = gs->vadj->value + gs->vadj->page_increment;
+		if (val > gs->vadj->upper - gs->vadj->page_size)
+			val = gs->vadj->upper - gs->vadj->page_size;
+		gtk_adjustment_set_value(gs->vadj, val);
+			break;
 	case GDK_plus:
 		g_signal_emit_by_name(G_OBJECT (control->priv->zoomable),
 							  "zoom_in");
@@ -660,34 +696,6 @@ verb_ZoomToFitWidth_cb (BonoboUIComponent *uic, gpointer user_data,
 
 }
 
-static void
-set_auto_fit_items_sensitivity(GgvControl *control, gboolean sens)
-{
-	gchar *prop_val = sens?"1":"0";
-	Bonobo_UIContainer container;
-
-	container = bonobo_ui_component_get_container (control->priv->uic);
-	if (container == CORBA_OBJECT_NIL)
-		return;
-
-	bonobo_ui_component_freeze(control->priv->uic, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitNone",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitWidth",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_set_prop(control->priv->uic, "/commands/AutoFitPage",
-								 "sensitive", prop_val, NULL);
-	bonobo_ui_component_thaw(control->priv->uic, NULL);	
-}
-
-static void
-set_command_items_sensitivity(GgvControl *control, gboolean sens)
-{
-	set_auto_fit_items_sensitivity(control, sens);
-	sens = sens & !is_auto_fit_set(control);
-	set_zoom_items_sensitivity(control, sens);
-}
-
 static BonoboUIVerb ggv_control_verbs[] = {
 		BONOBO_UI_VERB ("ZoomIn",         verb_ZoomIn_cb),
 		BONOBO_UI_VERB ("ZoomOut",        verb_ZoomOut_cb),
@@ -714,8 +722,6 @@ static void
 ggv_control_create_ui (GgvControl *control)
 {
 	GgvControlClass *klass;
-	GdkPixbuf *pixbuf, *pixbuf_small;
-	gchar *pixbuf_xml;
 	int i;
 
 	g_return_if_fail (control != NULL);
@@ -731,29 +737,6 @@ ggv_control_create_ui (GgvControl *control)
 	bonobo_ui_component_freeze(control->priv->popup_uic, NULL);
 	bonobo_ui_util_set_ui(control->priv->popup_uic, NULL, "ggv-postscript-view-ui.xml", "GGV", NULL);
 	bonobo_ui_util_set_ui(control->priv->popup_uic, DATADIR, "ggv-control-ui.xml", "GGV", NULL);
-	
-	pixbuf = gdk_pixbuf_new_from_file(GNOMEICONDIR "/ggv/fitwidth.png", NULL);
-	if(pixbuf != NULL) {
-		pixbuf_xml = bonobo_ui_util_pixbuf_to_xml(pixbuf);
-		bonobo_ui_component_set_prop(control->priv->uic,
-									 "/Toolbar/GgvItems/GgvZoomItems/ZoomToFitWidth",
-									 "pixname", pixbuf_xml,
-									 NULL);
-		g_free(pixbuf_xml);
-		pixbuf_small = gdk_pixbuf_scale_simple(pixbuf, 18, 18, GDK_INTERP_TILES);
-		pixbuf_xml = bonobo_ui_util_pixbuf_to_xml(pixbuf_small);
-		bonobo_ui_component_set_prop(control->priv->uic,
-									 "/menu/View/Zoom Items Placeholder/Zoom/ZoomToFitWidth",
-									 "pixname", pixbuf_xml,
-									 NULL);
-		bonobo_ui_component_set_prop(control->priv->popup_uic,
-									 "/popups/button3/ZoomMenu/ZoomToFitWidth",
-									 "pixname", pixbuf_xml,
-									 NULL);
-		g_free(pixbuf_xml);
-		gdk_pixbuf_unref(pixbuf);
-		gdk_pixbuf_unref(pixbuf_small);
-	}
 
 	for(i = 0; zoom_level_items[i].path; i++) {
 		bonobo_ui_component_add_listener(control->priv->uic,
@@ -770,7 +753,7 @@ ggv_control_create_ui (GgvControl *control)
 									 "AutoFitPage",
 									 listener_AutoFitMode_cb, control);
 
-	set_command_items_sensitivity(control, ggv_postscript_view_get_page_count(control->priv->ps_view) > 0);
+	set_command_items_sensitivity(control, ggv_document_get_page_count(GGV_DOCUMENT(control->priv->ps_view)) > 0);
 
 	sync_command_state(control);
 
@@ -793,7 +776,7 @@ ggv_control_create_ui (GgvControl *control)
 
 		hbox = gtk_hbox_new(FALSE, 2);
 		gtk_widget_show(hbox);
-		zoom_image = gtk_image_new_from_file(GNOMEICONDIR "/ggv/zoom.xpm");
+		zoom_image = gtk_image_new_from_stock(GGV_ZOOM, GTK_ICON_SIZE_LARGE_TOOLBAR);
 		gtk_widget_show(zoom_image);
 		gtk_box_pack_start(GTK_BOX(hbox), zoom_image, FALSE, TRUE, 0);
 		zoom_adj = gtk_adjustment_new(ggv_postscript_view_get_zoom_factor(control->priv->ps_view)*100.0,
@@ -843,9 +826,9 @@ ggv_control_unset_ui_container (GgvControl *control)
 	g_return_if_fail (control != NULL);
 	g_return_if_fail (GGV_IS_CONTROL (control));
 
-	ggv_postscript_view_unset_ui_container (control->priv->ps_view);
-
 	bonobo_ui_component_unset_container (control->priv->uic, NULL);
+
+	ggv_postscript_view_unset_ui_container (control->priv->ps_view);
 }
 
 static void
@@ -866,8 +849,9 @@ ggv_control_activate (BonoboControl *object, gboolean state)
 			ggv_control_set_ui_container (control, ui_container);
 			bonobo_object_release_unref (ui_container, NULL);
 		}
-	} else
+	} else {
 		ggv_control_unset_ui_container (control);
+	}
 
 	if (BONOBO_CONTROL_CLASS (ggv_control_parent_class)->activate)
 		BONOBO_CONTROL_CLASS (ggv_control_parent_class)->activate (object, state);
