@@ -184,12 +184,14 @@ ggv_window_remove_control(GgvWindow *win)
                 bonobo_object_release_unref(win->pb, NULL);
                 win->pb = CORBA_OBJECT_NIL;
         }
+#if 0
         if(win->view) {
                 gtk_container_remove(GTK_CONTAINER(win->hbox),
                                      win->view);
                 gtk_widget_unref(win->view);
                 win->view = NULL;
         }
+#endif
         if(win->filename) {
                 g_free(win->filename);
                 win->filename = NULL;
@@ -477,7 +479,22 @@ verb_FileClose(BonoboUIComponent *uic, gpointer data, const char *cname)
         if(window_list->next != NULL)
                 ggv_window_close(win);
         else {
-                ggv_window_remove_control(win);
+                GNOME_GGV_PostScriptView ps_view;
+                CORBA_Environment ev;
+
+                CORBA_exception_init(&ev);
+                ps_view = (GNOME_GGV_PostScriptView)
+                        Bonobo_Unknown_queryInterface(win->control,
+                                                      "IDL:GNOME/GGV/PostScriptView:1.0",
+                                                      &ev);
+                if(!BONOBO_EX(&ev)) {
+                        GNOME_GGV_PostScriptView_close(ps_view, &ev);
+                        if(BONOBO_EX(&ev))
+                                g_warning("Could not invoke reload method.");
+                        bonobo_object_release_unref(ps_view, &ev);
+                }
+                CORBA_exception_free(&ev);                
+                ggv_window_set_watch_file(win, FALSE);
         }
 }
 
@@ -745,11 +762,34 @@ ggv_window_destroy(GtkObject *object)
 
         win = GGV_WINDOW(object);
 
+        if(win->ctlframe != NULL) {
+                bonobo_object_unref(BONOBO_OBJECT(win->ctlframe));
+                win->ctlframe = NULL;
+        }
+        if(win->control != CORBA_OBJECT_NIL) {
+                bonobo_object_release_unref(win->control, NULL);
+                win->control = CORBA_OBJECT_NIL;
+        }
         if(win->uic) {
                 bonobo_object_unref(win->uic);
                 win->uic = NULL;
         }
-        ggv_window_remove_control(win);
+        if(win->popup_uic) {
+                bonobo_object_unref(win->popup_uic);
+                win->popup_uic = NULL;
+        }
+        if(win->pb) {
+                bonobo_object_release_unref(win->pb, NULL);
+                win->pb = CORBA_OBJECT_NIL;
+        }
+        if(win->filename) {
+                g_free(win->filename);
+                win->filename = NULL;
+        }
+        if(0 != win->timer_tag) {
+                gtk_timeout_remove(win->timer_tag);
+                win->timer_tag = 0;
+        }
 
         window_list = g_list_remove(window_list, win);
 
@@ -885,14 +925,31 @@ ggv_window_new(void)
 {
         GgvWindow *win;
 	BonoboUIContainer *ui_container;
-	BonoboUIComponent *uic;
+	BonoboUIComponent *uic, *popup_uic;
         CORBA_Environment ev;
+        Bonobo_Control control;
+        GtkWidget *widget;
+        Bonobo_PropertyBag pb;
+        gchar *mask;
 
 	static const GtkTargetEntry drag_types[] = {
 		{ "text/uri-list", 0, TARGET_URI_LIST }
 	};
 
+        /* get the control */
+        CORBA_exception_init(&ev);
+        control = bonobo_get_object(GGV_POSTSCRIPT_VIEWER_CONTROL_IID,
+                                    "Bonobo/Control", &ev);
+        if(BONOBO_EX(&ev) || control == CORBA_OBJECT_NIL) {
+                g_warning("Could not get GGV control: '%s'",
+                          bonobo_exception_get_text(&ev));
+                CORBA_exception_free(&ev);
+                return NULL;
+        }
+        CORBA_exception_free(&ev);
+
 	win = GGV_WINDOW(g_object_new(GGV_TYPE_WINDOW, "win_name", "ggv", "title", _("GGV: no document loaded"), NULL));
+        win->control = control;
 
         win->current_page = -2;
         win->show_toolbar = ggv_toolbar;
@@ -958,6 +1015,64 @@ ggv_window_new(void)
 	bonobo_control_frame_set_autoactivate(win->ctlframe, FALSE);
 	g_signal_connect(G_OBJECT(win->ctlframe), "activate_uri",
                          (GtkSignalFunc)control_frame_activate_uri, NULL);
+	bonobo_control_frame_bind_to_control(win->ctlframe, control, NULL);
+	widget = bonobo_control_frame_get_widget(win->ctlframe);
+	gtk_widget_show(widget);
+        gtk_box_pack_start(GTK_BOX(win->hbox), widget,
+                           TRUE, TRUE, 0);
+	bonobo_control_frame_control_activate(win->ctlframe);
+
+        /* now get the control's property bag */
+        pb = bonobo_control_frame_get_control_property_bag(win->ctlframe, NULL);
+        if(pb == CORBA_OBJECT_NIL) {
+                g_warning("Control does not have any properties.");
+        }
+        else {
+                /* TODO: set initial status & title */
+                mask =  "Bonobo/Property:change:page,"
+                        "Bonobo/Property:change:page_count,"
+                        "Bonobo/Property:change:width,"
+                        "Bonobo/Property:change:height,"
+                        "Bonobo/Property:change:title,"
+                        "Bonobo/Property:change:status";
+                bonobo_ui_component_set_prop(win->uic,
+                                             "/commands/FileReload",
+                                             "sensitive",
+                                             (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
+                                             NULL);
+                bonobo_ui_component_set_prop(win->uic,
+                                             "/commands/FileClose",
+                                             "sensitive",
+                                             (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
+                                             NULL);
+                bonobo_event_source_client_add_listener(pb,
+                                                        (BonoboListenerCallbackFn)control_property_changed_handler,
+                                                        mask, NULL, win);
+                win->pb = pb;
+        }
+
+        /* merge our items in the control's popup menu */
+        popup_uic = bonobo_control_frame_get_popup_component(win->ctlframe, NULL);
+        if(popup_uic == NULL) {
+                g_warning("Control does not have a popup component.");
+        }
+        else {
+                bonobo_ui_util_set_ui(popup_uic, DATADIR, "ggv-ui.xml",
+                                      "GGV", NULL);
+                bonobo_ui_component_add_listener(popup_uic, "SettingsShowMenus",
+                                                 listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "SettingsShowSidebar",
+                                                 listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "SettingsShowToolbar",
+                                                 listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "SettingsShowStatusbar",
+                                                 listener_SettingsShow, win);
+                bonobo_ui_component_add_listener(popup_uic, "ViewFullscreen",
+                                                 listener_ViewFullscreen, win);
+                win->popup_uic = popup_uic;
+                sync_settings_popup_items(win);
+        }
+
 	/* set default geometry */
         gtk_widget_set_usize(GTK_WIDGET(win),
                              ggv_default_width, ggv_default_height);
@@ -972,8 +1087,6 @@ ggv_window_new(void)
                                 (GConfClientNotifyFunc)prefs_changed,
                                 win, NULL, NULL);
 
-        set_file_cmds_sensitivity(win, FALSE);
-
 	return GTK_WIDGET(win);
 }
 
@@ -983,11 +1096,6 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
         CORBA_Environment ev;
         Bonobo_PersistFile pf;
         gchar *title, *unescaped, *utf8_title;
-	BonoboUIComponent *popup_uic;
-        Bonobo_Control control;
-        GtkWidget *widget;
-        Bonobo_PropertyBag pb;
-        gchar *mask;
         GnomeVFSFileInfo info;
 
         g_return_val_if_fail(win != NULL, FALSE);
@@ -1005,92 +1113,20 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
                 return FALSE;
 
         CORBA_exception_init(&ev);
-
-        if(NULL == win->control) {
-                control = bonobo_get_object(GGV_POSTSCRIPT_VIEWER_CONTROL_IID,
-                                            "Bonobo/Control", &ev);
-                if(BONOBO_EX(&ev) || control == CORBA_OBJECT_NIL) {
-                        g_warning("Could not get GGV control: '%s'",
-                                  bonobo_exception_get_text(&ev));
-                        CORBA_exception_free(&ev);
-                        return FALSE;
-                }
+        pf = Bonobo_Unknown_queryInterface(win->control,
+                                           "IDL:Bonobo/PersistFile:1.0", &ev);
+        if(BONOBO_EX(&ev) || pf == CORBA_OBJECT_NIL) {
                 CORBA_exception_free(&ev);
-
-                /* insert the control */
-                win->control = control;
-                bonobo_control_frame_bind_to_control(win->ctlframe, control,
-                                                     NULL);
-                widget = bonobo_control_frame_get_widget(win->ctlframe);
-                win->view = widget;
-                gtk_widget_ref(win->view);
-                gtk_widget_show(widget);
-                gtk_box_pack_start(GTK_BOX(win->hbox), widget,
-                                   TRUE, TRUE, 0);
-                bonobo_control_frame_control_activate(win->ctlframe);
-                
-                /* now get the control's property bag */
-                pb = bonobo_control_frame_get_control_property_bag
-                        (win->ctlframe, NULL);
-                if(pb == CORBA_OBJECT_NIL) {
-                        g_warning("Control does not have any properties.");
-                }
-                else {
-                        /* TODO: set initial status & title */
-                        mask =  "Bonobo/Property:change:page,"
-                                "Bonobo/Property:change:page_count,"
-                                "Bonobo/Property:change:width,"
-                                "Bonobo/Property:change:height,"
-                                "Bonobo/Property:change:title,"
-                                "Bonobo/Property:change:status";
-                        bonobo_ui_component_set_prop
-                                (win->uic, "/commands/FileReload", "sensitive",
-                                 (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
-                                 NULL);
-                        bonobo_ui_component_set_prop
-                                (win->uic, "/commands/FileClose", "sensitive",
-                                 (bonobo_pbclient_get_long(pb, "page", NULL) >= 0)?"1":"0",
-                                 NULL);
-                        bonobo_event_source_client_add_listener
-                                (pb,
-                                 (BonoboListenerCallbackFn)control_property_changed_handler,
-                                 mask, NULL, win);
-                        win->pb = pb;
-                }
-                
-                /* merge our items in the control's popup menu */
-                popup_uic = bonobo_control_frame_get_popup_component
-                        (win->ctlframe, NULL);
-                if(popup_uic == NULL) {
-                        g_warning("Control does not have a popup component.");
-                }
-                else {
-                        bonobo_ui_util_set_ui(popup_uic, DATADIR, "ggv-ui.xml",
-                                              "GGV", NULL);
-                        bonobo_ui_component_add_listener
-                                (popup_uic, "SettingsShowMenus",
-                                 listener_SettingsShow, win);
-                        bonobo_ui_component_add_listener
-                                (popup_uic, "SettingsShowSidebar",
-                                 listener_SettingsShow, win);
-                        bonobo_ui_component_add_listener
-                                (popup_uic, "SettingsShowToolbar",
-                                 listener_SettingsShow, win);
-                        bonobo_ui_component_add_listener
-                                (popup_uic, "SettingsShowStatusbar",
-                                 listener_SettingsShow, win);
-                        bonobo_ui_component_add_listener
-                                (popup_uic, "ViewFullscreen",
-                                 listener_ViewFullscreen, win);
-                        win->popup_uic = popup_uic;
-                        sync_settings_popup_items(win);
-                }
+                return FALSE;
         }
-                
-        /* store filename and set title */
+
+        Bonobo_PersistFile_load(pf, filename, &ev);
+        bonobo_object_release_unref(pf, NULL);
+
         if(win->filename)
                 g_free(win->filename);
         win->filename = g_strdup(filename);
+
         unescaped = gnome_vfs_unescape_string_for_display(win->filename);
         title = g_strconcat(_("GGV: "), unescaped, NULL);
         g_free(unescaped);
@@ -1098,17 +1134,22 @@ ggv_window_load(GgvWindow *win, const gchar *filename)
         g_free(title);
         gtk_window_set_title(GTK_WINDOW(win), utf8_title);
         g_free(utf8_title);
-        
-        pf = Bonobo_Unknown_queryInterface(win->control,
-                                           "IDL:Bonobo/PersistFile:1.0", &ev);
-        if(BONOBO_EX(&ev) || pf == CORBA_OBJECT_NIL) {
+
+        if(BONOBO_EX(&ev)) {
                 CORBA_exception_free(&ev);
                 return FALSE;
         }
-        Bonobo_PersistFile_load(pf, win->filename, &ev);
-        bonobo_object_release_unref(pf, NULL);
         CORBA_exception_free(&ev);
 
+        bonobo_ui_component_set_prop
+                (win->uic, "/commands/FileReload", "sensitive",
+                 (bonobo_pbclient_get_long(win->pb, "page", NULL) >= 0)?"1":"0",
+                 NULL);
+        bonobo_ui_component_set_prop
+                (win->uic, "/commands/FileClose", "sensitive",
+                 (bonobo_pbclient_get_long(win->pb, "page", NULL) >= 0)?"1":"0",
+                 NULL);
+
         ggv_window_set_watch_file(win, ggv_watch_doc);
 
         return TRUE;
